/** <module> NXT Mindstroms - Actions Dummy (does nothing, describes actions only).
@authors: 	API: Piotr Hoownia
			Definitions: Marcin Zikowski

Motors: 'A', 'B', 'C'.
Sensors ports: 'S1', 'S2', 'S3', 'S4'.
*/

:- module(nxt_actions_dummy,[
			nxt_actions_connection_open/0,
			nxt_actions_connection_close/0,
			nxt_actions_motor_forward/2,
			nxt_actions_motor_backward/2,
			nxt_actions_motor_get_speed/2,
			nxt_actions_motor_stop/1,
			nxt_actions_motor_rotate/3,
			nxt_actions_touch_sensor/2,
			nxt_actions_sound_sensor/2,
			nxt_actions_light_sensor/2,
			nxt_actions_light_sensor_LED/2,
			nxt_actions_ultrasonic_sensor/2
		       ]).
			   
:- use_module(library(socket)).
:- dynamic nxt_actions_sockets/2.


polacz([],W,W).

polacz([X|Y],W,Wynik):-
polacz(Y,W,Wyj),
char_code(XY,X),
concat(XY,Wyj,Wynik).

%%	nxt_actions_open_connection
%
%	Opens connection via Bluetooth using iCommand.
%	nxt_actions_light_sensor(Port,Value)
%	nxt_actions_ultrasonic_sensor(Port,Value)
%	nxt_actions_sound_sensor(Port,Value) 
%	nxt_actions_motor_rotate(Motor,Speed,Angle)

nxt_actions_connection_open :-
        tcp_socket(Socket),
        tcp_connect(Socket, '127.0.0.1':2056),
        tcp_open_socket(Socket, ReadFd, WriteFd),
	
		assert(nxt_actions_sockets(ReadFd, WriteFd)),
		
		write_canonical(WriteFd, 'nxt; open'),
		flush_output(WriteFd),
		read_line_to_codes(ReadFd, _),
		
		write_canonical(WriteFd, 'shortrep'),
		flush_output(WriteFd).

%%	nxt_actions_close_connection
%
%	Closes connection via Bluetooth using iCommand.

nxt_actions_connection_close :-
		nxt_actions_sockets(ReadFd, WriteFd),

		write_canonical(WriteFd, 'nxt; close'),
		flush_output(WriteFd),
		
		write_canonical(WriteFd, 'bye'),
		flush_output(WriteFd),
		
		close(ReadFd),
		close(WriteFd),
		
		retractall(nxt_actions_sockets(_,_)).

%%	nxt_actions_motor_forward(+Motor,+Speed).
%
%	Rotates motor forward at specified speed.

nxt_actions_motor_forward(Motor,Speed) :-
		nxt_actions_sockets(ReadFd, WriteFd),
		string_concat('motor; setForward;', Motor, A),
		string_concat(A, ';', B),
		string_concat(B, Speed, C),
		write_canonical(WriteFd, C),
		flush_output(WriteFd),
		read_line_to_codes(ReadFd, _).

%%	nxt_actions_motor_forward(+Motor,+Speed).
%
%	Rotates motor backward at specified speed.

nxt_actions_motor_backward(Motor,Speed) :-
		nxt_actions_sockets(ReadFd, WriteFd),
		string_concat('motor; setReverse;', Motor, A),
		string_concat(A, ';', B),
		string_concat(B, Speed, C),
		write_canonical(WriteFd, C),
		flush_output(WriteFd),
		read_line_to_codes(ReadFd, _).

%%	nxt_actions_motor_speed(+Motor,+Speed).
%
%	Gets speed of the motor.

nxt_actions_motor_get_speed(Motor,Speed) :-
		nxt_actions_sockets(ReadFd, WriteFd),
		string_concat('motor; getRotationCount;', Motor, A),
		write_canonical(WriteFd, A),
		flush_output(WriteFd),
		read_line_to_codes(ReadFd, Speedtemp),
		polacz(Speedtemp,'',Speed).
		

%%	nxt_motor_stop(+Motor).
%
%	Stops motor.

nxt_actions_motor_stop(Motor) :-
		nxt_actions_sockets(ReadFd, WriteFd),
		string_concat('motor; stop;', Motor, A),
		string_concat(A, '; false', B),
		write_canonical(WriteFd, B),
		flush_output(WriteFd),
		read_line_to_codes(ReadFd, _).

%%	nxt_actions_motor_rotate(+Motor,+Speed,+Angle).
%
%	Rotates motor at specified speed.
%	Forward if Angle is positive, backward if Angle is negative.
%	Motor will stop, when specified revolution (Angle in degrees) is reached.

nxt_actions_motor_rotate(Motor,Speed,Angle) :-
		nxt_actions_sockets(ReadFd, WriteFd),
		string_concat('motor; goTo;', Motor, A),
		string_concat(A, ';', B),
		string_concat(B, Speed, C),
		string_concat(C, ';', D),
		string_concat(D, Angle, E),
		string_concat(E, '; false', F),
		write_canonical(WriteFd, F),
		flush_output(WriteFd),
		read_line_to_codes(ReadFd, _).

%%	nxt_actions_touch_sensor(+Port,-Value).
%
%	Gets touch sensor reading.
%	Returns 1 if pressed, 0 otherwise.

nxt_actions_touch_sensor(Port,Value) :-
		nxt_actions_sockets(ReadFd, WriteFd),
		string_concat('sensor; setTouch;', Port, A),
		write_canonical(WriteFd, A),
		flush_output(WriteFd),
		read_line_to_codes(ReadFd, _),
		
		nxt_actions_sockets(ReadFd, WriteFd),
		string_concat('sensor; getValue;', Port, B),
		write_canonical(WriteFd, B),
		flush_output(WriteFd),
		read_line_to_codes(ReadFd, Valuetmp),
		polacz(Valuetmp,'',Value).

%%	nxt_actions_sound_sensor(+Port,-Value).
%
%	Gets sound sensor reading.

nxt_actions_sound_sensor(Port,Value) :-
		nxt_actions_sockets(ReadFd, WriteFd),
		string_concat('sensor; setSound;', Port, A),
		write_canonical(WriteFd, A),
		flush_output(WriteFd),
		read_line_to_codes(ReadFd, _),
		
		nxt_actions_sockets(ReadFd, WriteFd),
		string_concat('sensor; getValue;', Port, B),
		write_canonical(WriteFd, B),
		flush_output(WriteFd),
		read_line_to_codes(ReadFd, Valuetmp),
		polacz(Valuetmp,'',Value).

%%	nxt_actions_light_sensor(+Port,-Value).
%
%	Gets light sensor reading.

nxt_actions_light_sensor(Port,Value) :-
		nxt_actions_sockets(ReadFd, WriteFd),
		string_concat('sensor; setLight;', Port, A),
		string_concat(A, ';', B),
		string_concat(B, 'false', C),
		write_canonical(WriteFd,C),
		flush_output(WriteFd),
		read_line_to_codes(ReadFd, _),
		
		nxt_actions_sockets(ReadFd, WriteFd),
		string_concat('sensor; getValue;', Port, D),
		write_canonical(WriteFd, D),
		flush_output(WriteFd),
		read_line_to_codes(ReadFd, Valuetmp),
		polacz(Valuetmp,'',Value).
		
%%	nxt_actions_light_sensor_LED(+Port,+Setting).
%
%	Sets the LED on if Setting is activate or off if passivate.

nxt_actions_light_sensor_LED(Port,Setting) :-
		nxt_actions_sockets(ReadFd, WriteFd),
		string_concat('sensor; setSonarContinuous;', Port, A),
		write_canonical(WriteFd, A),
		flush_output(WriteFd),
		read_line_to_codes(ReadFd, _),

		nxt_actions_sockets(ReadFd, WriteFd),
		string_concat('sensor; setSonarOff;', Port, A),
		write_canonical(WriteFd, A),
		flush_output(WriteFd),
		read_line_to_codes(ReadFd, _).
	

%%	nxt_actions_ultrasonic_sensor(+Port,-Value).
%
%	Gets ultrasonic sensor reading.

nxt_actions_ultrasonic_sensor(Port,Value) :-
		nxt_actions_sockets(ReadFd, WriteFd),
		string_concat('sensor; setSonar;', Port, A),
		write_canonical(WriteFd, A),
		flush_output(WriteFd),
		read_line_to_codes(ReadFd, _),
		
		nxt_actions_sockets(ReadFd, WriteFd),
		string_concat('sensor; getSonarValue;', Port, B),
		write_canonical(WriteFd, B),
		flush_output(WriteFd),
		read_line_to_codes(ReadFd, Valuetmp),
		polacz(Valuetmp,'',Value).
		
%%	nxt_actions_start_program(+File)
%
%	Starts a Lego executable file on the NXT.
%	DOES NOT WORK PROPERLY!!

nxt_actions_start_program(File) :-
		nxt_actions_sockets(ReadFd, WriteFd),
		string_concat('nxt; startProgram;', File, A),
		write_canonical(WriteFd, A),
		flush_output(WriteFd),
		read_line_to_codes(ReadFd, _).
		
%%	nxt_actions_stop_program.
%
%	Stops the currently running Lego executable on the NXT.
%	DOES NOT WORK PROPERLY!!

nxt_actions_stop_program :-
		nxt_actions_sockets(ReadFd, WriteFd),
		string_concat('nxt; stopProgram', '', A),
		write_canonical(WriteFd, A),
		flush_output(WriteFd),
		read_line_to_codes(ReadFd, _).
		
%%	nxt_actions_play_sound_file(+File,+Repeat).
%
%	Plays a sound file from the NXT.
%	Files use the .rso extension.
%	The filename is not case sensitive.
%	Plays sound once if Repeat is true
%	or till nxt_actions_stop_sound_playback is used if Repeat is
%	false.

nxt_actions_play_sound_file(File,True) :-
		nxt_actions_sockets(ReadFd, WriteFd),
		string_concat('nxt; playSoundFile;', File, A),
		string_concat(A,';', B),
		string_concat(B,True, C),
		write_canonical(WriteFd, C),
		flush_output(WriteFd),
		read_line_to_codes(ReadFd, _).

%%	nxt_actions_stop_sound_playback.
%
%	Stops a sound file that has been playing repeating.

nxt_actions_stop_sound_playback :-
		nxt_actions_sockets(ReadFd, WriteFd),
		string_concat('nxt; stopSound', '', A),
		write_canonical(WriteFd, A),
		flush_output(WriteFd),
		read_line_to_codes(ReadFd, _).
		
%%	nxt_actions_play_tone(Frequency,Duration).
%
%	Plays tone at specified frequency.

nxt_actions_play_tone(Frequency,Duration) :-
		nxt_actions_sockets(ReadFd, WriteFd),
		string_concat('nxt; playTone;', Frequency, A),
		string_concat(A,';', B),
		string_concat(B,Duration, C),
		write_canonical(WriteFd, C),
		flush_output(WriteFd),
		read_line_to_codes(ReadFd, _).
		
%%	nxt_actions_get_brick_name(-Name).
%
%	Gets the name of the brick.
%	DOES NOT WORK PROPERLY!!

nxt_actions_get_brick_name(Name) :-
		nxt_actions_sockets(ReadFd, WriteFd),
		string_concat('nxt; getName', '', A),
		write_canonical(WriteFd, A),
		flush_output(WriteFd),
		read_line_to_codes(ReadFd, Valuetmp),
		polacz(Valuetmp,'',Name).
		
%%	nxt_actions_voltage_millivolt(-Voltage).
%
%	Returns battery voltage in mV.

nxt_actions_voltage_millivolt(Voltage) :-
		nxt_actions_sockets(ReadFd, WriteFd),
		string_concat('sensor; batteryLevel', Voltage, B),
		write_canonical(WriteFd, B),
		flush_output(WriteFd),
		read_line_to_codes(ReadFd, Valuetmp),
		polacz(Valuetmp,'',Voltage).
		
nxt_actions_motor_reset_for_speed(Motor) :-
		nxt_actions_sockets(ReadFd, WriteFd),
		string_concat('motor; resetRotationCount', '', A),
		write_canonical(WriteFd, A),
		flush_output(WriteFd),
		read_line_to_codes(ReadFd, _).
		