/* Possible commands:

// ==================================================
// motor module
motor; getRotationCount; portNr
motor; resetRotationCount; portNr; relative
motor; setForward; portNr; power
motor; setReverse; portNr; power
motor; stop; portNr; brake
motor; breakOn; portNr
motor; breakOff; portNr
motor; goTo; portNr; power; tacho; brake

// ==================================================
Possible values args:

portNr: OUT_A, OUT_B, OUT_C
power: 0 - 100
relative: TRUE, FALSE
brake: TRUE, FALSE
tacho: degrees

// ==================================================
//sensor module
sonar; setTouch; portNr
sonar; setSound; portNr
sonar; setLight; portNr, active
sonar; setSonar; portNr
sonar; setRaw; portNr
sonar; getValue; portNr
sonar; lsGetStatus; portNr
sonar; getSonarValue; portNr
sonar; setSonarOff; portNr
sonar; setSonarSingleShot; portNr
sonar; setSonarContinuous; portNr
sonar; setSonarContinuousInterval; portNr, interval

// ==================================================
Possible values args:

portNr: IN_1, IN_2, IN_3, IN_4
active: TRUE, FALSE
interval: miliseconds

// ==================================================
//nxt module
nxt; open
nxt; close
nxt; batteryLevel
nxt; getName
nxt; startProgram; name
nxt; stopProgram
nxt; playTone;
nxt; playSoundFile; name, loop
nxt; stopSound;
// ==================================================
Possible values arg:

loop: TRUE, FALSE

// ==================================================

/* Possible replies
ok
ok: reply

err: not_enable_interface
err: unknown_module: [what]
err: unknown_command: [what]
err: wrong_params: [what]
*/

#include "../include/CommandNXT.h"
#include <fantom/iNXT.h>


// constructor
cCommandNXT::cCommandNXT()
{
	this->isOpen = false;
	shortReply = false;
}

// destructor
cCommandNXT::~cCommandNXT()
{
}

// Convert string type to int
int cCommandNXT::Str2Int(const string &str)
{
	stringstream ss(str);
	int n;
	ss >> n;
	
	return n;
}

// Convert int type to string
string cCommandNXT::Int2Str(int n)
{
	stringstream ss;
	ss << n;

	return ss.str();
}

// Convert str type to bool
bool cCommandNXT::Str2Bool(const string &str)
{
	stringstream ss(str);
	bool boolen;
	ss >> boolen;
	
	return boolen;
}

// enable short reply - special for few clients
void cCommandNXT::EnableShortReply()
{
	shortReply = true;
}

// change string to lowercase
string cCommandNXT::ToLowerCase(string str)
{
   for(int i = 0; i < str.length(); ++i)
      str[i] = tolower(str[i]);

   return str;
}

/*
// remove from string chars CR/LF
string cCommandNXT::Escape(string str)
{
   for(int i = 0; i < str.length(); ++i)
	   if(str[i] == 13 || str[i] == 10)
		   str[i] = '\0';

   return str;
}
*/

// Check output port
bool cCommandNXT::CheckOutPort(const string& port)
{
	if(port == "out_a" || port == "out_b" || port == "out_c" \
		|| port == "a" || port == "b" || port == "c" )
		return true;
	
	return false;
}

// Check input port
bool cCommandNXT::CheckInPort(const string& port)
{
	if(port == "in_1" || port == "in_2" || port == "in_3" || port == "in_4" \
		|| port == "s1" || port == "s2" || port == "s3" || port == "s4")
		return true;
	
	return false;
}

// Check range of power
bool cCommandNXT::CheckPower(const string& power)
{
	if(Str2Int(power) >= 0 && Str2Int(power) <= 100)
		return true;

	return false;
}

// Check boolean value
bool cCommandNXT::CheckBoolean(const string& boolen)
{
	if(boolen == "true" || boolen == "false")
		return true;

	return false;
}

// Check tacho
bool cCommandNXT::CheckTacho(const string& tacho)
{
	return false;
}

// Convert output port
int cCommandNXT::ConvertPortOut(const string& port)
{
	if(port == "out_a" || port == "a")
		return 0;

	else if(port == "out_b" || port == "b")
		return 1;

	else if(port == "out_c" || port == "c")
		return 2;

	return -1;
}

// Convert input port
int cCommandNXT::ConvertPortIn(const string& port)
{
	if(port == "in_1" || port == "s1")
		return 0;

	else if(port == "in_2" || port == "s2")
		return 1;

	else if(port == "in_3" || port == "s3")
		return 2;

	else if(port == "in_4" || port == "s4")
		return 3;

	return -1;
}

// choose the module and call appropriate
string cCommandNXT::RedirectCmd(vector<string>& cmds)
{

	if(cmds.empty())
		return "err: unknown_module";

	//motor module
	if(cmds.at(0) == "motor")
	{
		return MotorCmd(cmds);
	}

	//sensor module
	else if(cmds.at(0) == "sensor")
	{
		return SensorCmd(cmds);
	}

	//nxt module
	else if(cmds.at(0) == "nxt")
	{
		return NXTCmd(cmds);
	}

	//unknown module
	return "err: unknown_module: " + cmds.at(0);
}

// motor module
string cCommandNXT::MotorCmd(vector<string>& cmds)
{
	if(cmds.size() < 2)
		return "err: wrong_params";

	string cmd = cmds.at(1);
	string reply = "ok";

	int countArgs = cmds.size();

	if(this->isOpen == false)
		return "err: not_enable_interface";

	// GetRotationCount
	if(cmd == "getrotationcount")
	{
		if(countArgs < 3 || !CheckOutPort( cmds.at(2) ))
			return "err: wrong_params";
		
		string res = Int2Str( NXT::Motor::GetRotationCount(ConvertPortOut(cmds.at(2))) );
		reply =  (shortReply == false)? reply + res : res ;
	}

	// ResetRotationCount
	else if(cmd == "resetrotationcount")
	{
		if(countArgs < 3 || !CheckOutPort( cmds.at(2) ))
			return "err: wrong_params";

		NXT::Motor::ResetRotationCount(ConvertPortOut(cmds.at(2)), false);
	}

	// SetForward
	else if(cmd == "setforward")
	{
		if(countArgs < 4 || !CheckOutPort( cmds.at(2) ) || !CheckPower(cmds.at(3)))
			return "err: wrong_params";

		NXT::Motor::SetForward(ConvertPortOut(cmds.at(2)), Str2Int(cmds.at(3)));
	}

	// SetReverse
	else if(cmd == "setreverse")
	{
		if(countArgs < 4 || !CheckOutPort( cmds.at(2) ) || !CheckPower(cmds.at(3)))
			return "err: wrong_params";

		NXT::Motor::SetReverse(ConvertPortOut(cmds.at(2)), Str2Int(cmds.at(3)));
	}

	// Stop
	else if(cmd == "stop")
	{
		if(countArgs < 4 || !CheckOutPort( cmds.at(2) ) || !CheckBoolean(cmds.at(3)))
			return "err: wrong_params";

		NXT::Motor::Stop(ConvertPortOut(cmds.at(2)), Str2Bool(cmds.at(3)));
	}

	// BrakeOn
	else if(cmd == "brakeon")
	{
		if(countArgs < 3 || !CheckOutPort( cmds.at(2) ))
			return "err: wrong_params";

		NXT::Motor::BrakeOn(ConvertPortOut(cmds.at(2)));
	}

	// BrakeOff
	else if(cmd == "brakeoff")
	{
		if(countArgs < 3 || !CheckOutPort( cmds.at(2) ))
			return "err: wrong_params";

		NXT::Motor::BrakeOff(ConvertPortOut(cmds.at(2)));
	}

	// GoTo
	else if(cmd == "goto")	
	{
		if(countArgs < 6 || !CheckOutPort( cmds.at(2) ) || !CheckPower(cmds.at(3)))
			return "err: wrong_params";

		NXT::Motor::GoTo(ConvertPortOut(cmds.at(2)), Str2Int(cmds.at(3)), Str2Int(cmds.at(4)), false );
	}

	// unknown command
	else
	{
		return "err: unknown_command: >>" + cmd;
	}
	
	return reply;
}

// sensor module
string cCommandNXT::SensorCmd(vector<string>& cmds)
{
	if(cmds.size() < 2)
		return "err: wrong_params";

	std::string cmd = cmds.at(1);
	std::string reply = "ok";

	int countArgs = cmds.size();

	if(this->isOpen == false)
		return "err: not_enable_interface";

	bool wasSet=false;

	// SetTouch
	if(cmd == "settouch")
	{
		if(countArgs < 3 || !CheckInPort( cmds.at(2) ))
			return "err: wrong_params";

		NXT::Sensor::SetTouch(ConvertPortIn( cmds.at(2) ));
		wasSet=true;
	}

	// SetSound
	if(cmd == "setsound")
	{
		if(countArgs < 3 || !CheckInPort( cmds.at(2) ))
			return "err: wrong_params";

		NXT::Sensor::SetSound(ConvertPortIn( cmds.at(2) ));
		wasSet=true;
	}

	// SetLight
	if(cmd == "setlight")
	{
		if(countArgs < 4 || !CheckInPort( cmds.at(2) ) || !CheckBoolean( cmds.at(3) ))
			return "err: wrong_params";

		NXT::Sensor::SetLight(ConvertPortIn( cmds.at(2) ), false );
		wasSet=true;
	}

	// SetSonar
	if(cmd == "setsonar")
	{
		if(countArgs < 3 || !CheckInPort( cmds.at(2) ))
			return "err: wrong_params";

		NXT::Sensor::SetSonar(ConvertPortIn( cmds.at(2) ));
		wasSet=true;
	}

	// SetRaw
	if(cmd == "setraw")
	{
		if(countArgs < 3 || !CheckInPort( cmds.at(2) ))
			return "err: wrong_params";

		NXT::Sensor::SetRaw(ConvertPortIn( cmds.at(2) ));
		wasSet=true;
	}

	// GetValue
	if(cmd == "getvalue")
	{
		if(countArgs < 3 || !CheckInPort( cmds.at(2) ))
			return "err: wrong_params";

		string res = Int2Str(NXT::Sensor::GetValue( ConvertPortIn(cmds.at(2) )));
		reply = (shortReply == false)? reply + res : res; 
		wasSet=true;
	}

	// LSGetStatus
	if(cmd == "lsgetstatus")
	{
		if(countArgs < 3 || !CheckInPort( cmds.at(2) ))
			return "err: wrong_params";

		string res = Int2Str(NXT::Sensor::LSGetStatus(ConvertPortIn( cmds.at(2) )));
		reply = (shortReply == false)? reply + res : res;  
		wasSet=true;
	}

	// GetSonarValue
	if(cmd == "getsonarvalue")
	{
		if(countArgs < 3 || !CheckInPort( cmds.at(2) ))
			return "err: wrong_params";

		string res = Int2Str(NXT::Sensor::GetSonarValue(ConvertPortIn( cmds.at(2) )));
		reply = (shortReply == false)? reply + res : res; 
		wasSet=true;
	}

	// SetSonarOff
	if(cmd == "setsonaroff")
	{
		if(countArgs < 3 || !CheckInPort( cmds.at(2) ))
			return "err: wrong_params";

		NXT::Sensor::SetSonarOff(ConvertPortIn( cmds.at(2) ));
		wasSet=true;
	}

	// SetSonarSingleShot
	if(cmd == "setsonarsingleshot")
	{
		if(countArgs < 3 || !CheckInPort( cmds.at(2) ))
			return "err: wrong_params";

		NXT::Sensor::SetSonarSingleShot(ConvertPortIn( cmds.at(2) ));
		wasSet=true;
	}

	// SetSonarContinuous
	if(cmd == "setsonarcontinuous")
	{
		if(countArgs < 3 || !CheckInPort( cmds.at(2) ))
			return "err: wrong_params";

		NXT::Sensor::SetSonarContinuous(ConvertPortIn( cmds.at(2) ));
		wasSet=true;
	}

	// SetSonarContinuousInterval
	if(cmd == "setsonarcontinuousinterval")
	{
		if(countArgs < 4 || !CheckInPort( cmds.at(2) ))
			return "err: wrong_params";

		NXT::Sensor::SetSonarContinuousInterval(ConvertPortIn( cmds.at(2) ), Str2Int( cmds.at(3) ));
		wasSet=true;
	}

	// unknown command
	if(!wasSet)
	{
		return "err: unknown_command: " + cmd;
	}
	
	return reply;
}

// nxt module
string cCommandNXT::NXTCmd(vector<string>& cmds)
{
	if(cmds.size() < 2)
		return "err: wrong_params";

	string cmd = cmds.at(1);
	string reply = "ok";

	int countArgs = cmds.size();

	// Pair BT
	if(cmd == "pair")
	{
		nFANTOM100::tStatus status;
		ViChar *pairedResourceName = new ViChar[cmds.at(4).size()];

		for(int i = 0; i < cmds.at(4).size(); ++i)
			pairedResourceName[i] = cmds.at(4)[i];

		if(countArgs < 5)
			return "err: wrong_params";

		nFANTOM100::iNXT::pairBluetooth(reinterpret_cast<ViConstString>(&cmds.at(2)), reinterpret_cast<ViConstString>(&cmds.at(3)), pairedResourceName, status);

		if(!status.isNotFatal())
			return "err";
	}

	// Unpair BT
	if(cmd == "unpair")
	{
		nFANTOM100::tStatus status;

		if(countArgs < 3)
			return "err: wrong_params";

		nFANTOM100::iNXT::unpairBluetooth(reinterpret_cast<ViConstString>(&cmds.at(2)), status);

		if(!status.isNotFatal())
			return "err";
	}

	bool open = false;

	if(!(this->isOpen == false && cmd != "open"))
	{
		open=true;
	}

	if(this->isOpen == false && cmd != "openbt" && open == false)
	{
		return "err: not_enable_interface";
	}

	// Open
	if(cmd == "open")
	{
		if(!NXT::Open())
			return "err";
		
		this->isOpen = true;
	}

	// OpenBT
	else if(cmd == "openbt")
	{
		if(!NXT::OpenBT())
			return "err";
		
		this->isOpen = true;
	}

	// Close
	else if(cmd == "close")
	{
		NXT::Close();

		this->isOpen = false;
		reply = "-1-";
	}

	// BatteryLevel
	else if(cmd == "batterylevel")
	{
		string res = Int2Str( NXT::BatteryLevel() );
		reply = (shortReply == false)? reply + res : res; 
	}

	// GetName
	else if(cmd == "getname")
	{
		string res = NXT::GetName();
		reply = (shortReply == false)? reply + res : res;
	}

	// StartProgram
	else if(cmd == "startprogram")
	{
		if(countArgs < 3)
			return "err: wrong_params";

		NXT::StartProgram( cmds.at(2) );
	}

	// StopProgram
	else if(cmd == "stopprogram")
	{
		NXT::StopProgram();
	}

	// PlayTone
	else if(cmd == "playtone")
	{
		if(countArgs < 4)
			return "err: wrong_params";

		NXT::PlayTone(Str2Int( cmds.at(2) ), Str2Int( cmds.at(3) ));
	}

	// PlaySoundFile
	else if(cmd == "playsoundfile")
	{
		if(countArgs < 4 || !CheckBoolean( cmds.at(3) ))
			return "err: wrong_params";

		NXT::PlaySoundFile( cmds.at(2), Str2Bool( cmds.at(3) ));
	}

	// StopSound
	else if(cmd == "stopsound")
	{
		NXT::StopSound();
	}

	// unknown command
	else
	{
		return "err: unknown_command: " + cmd;
	}
	
	return reply;
}

// parse the request and to operate it
string cCommandNXT::CallCmd(string& obj)
{
	string str;
	vector<string> cmds;

	istringstream sstr(obj);

	while(getline(sstr, str, ';'))
	{
		istringstream sstrSub(str);
		sstrSub >> str;
		cmds.push_back(ToLowerCase(str));
	}
		
	return RedirectCmd(cmds);
}
